/************************************************************************
* \brief: This code implements an array, which  is resizable. The memory 
*         is continuous and therefore must be copied each time the
*         resize function is called
*														
* (c) copyright by Jrn Fischer							
*                                                         
*															
* @autor: Prof.Dr.Jrn Fischer									
* @email: j.fischer@hs-mannheim.de									
*                                                                     
* @file : Vector.h                                                    
*************************************************************************/

#include <stdio.h>
#include <stdlib.h>

#ifndef VECTOR_H
#define VECTOR_H

/*******************************************************************
* The class Vector enables the user to create a Vector. The Function
* resize enables the programmer to resize the array without
* distroying data.
*********************************************************************/
class  Vector
{
private:
	double *value;	
	int m_size;
	
public:
	// constructor initializes the variables
	Vector(int startSize)
	{
		m_size=0;
		value = NULL;
		resize(startSize);
	}
	// destructor frees the memory
	~Vector(){
		if (value!=NULL){
			delete[] value;
		}
	}

/************************************************************************
* function: resize
* enables the programmer to resize the vector without loosing data
*************************************************************************/
	void resize(int size)
	{
		double *newValue = new double[size];
		if (value!=NULL && newValue!=NULL){
			for (int t=0; (t<size) && (t<m_size); t++){
				newValue[t] = value[t];
			}
			delete[] value;
		}
		value = newValue;
		m_size = size;
	}
/************************************************************************
* function: &operator[] is called each time an object of this class is used
*           with the brackets
*************************************************************************/
	// the & deals as reference, so that data may be read and written into the array
	double &operator[] (int index){ 
		if (index<0){
			printf("Index ist negativ\n");
			return value[0];
		}
		if (index>m_size-1){
			resize(index+1024);
		}
		return value[index];
	}
};

#endif


