/************************************************************************
* \brief: This code demonstrates the use of dynamical arrays
*         and the respectation of array boundaries
*														
* (c) copyright by Jrn Fischer							
*                                                         
*															
* @autor: Prof.Dr.Jrn Fischer									
* @email: j.fischer@hs-mannheim.de									
*                                                                     
* @file : main.cpp                                                    
*************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "FArray.h"
#include "Vector.h"
#define MAX_SIZE 1000000
#define check(size) (size<MAX_SIZE)?size:arrayOutOfBound(MAX_SIZE-1)

/************************************************************************
* function: arrayOutOfBound
*************************************************************************/
int arrayOutOfBound(int size)
{
	printf("ERROR: ArrayOutOfBound!\n");
	return size;
}

/************************************************************************
* function: main
*************************************************************************/
int main()
{
	clock_t beginTimer, endTimer;

	static double test1[MAX_SIZE];
	FArray test2(10000000);
	Vector test3(1000); // mit Startgre 1000

	// --- Test 0 -------------------------------------------------------------------------
	printf("Test 0: plain C-Array!\n"); 
	beginTimer = clock();      // start the timer  // normalerweise CLK_TCK
	for (int t=0;t<1000000;t++){
		test1[t]=5;
	}
	endTimer = clock();
	printf("Timer: %f seconds.\n",((double)(endTimer - beginTimer)/(double)CLOCKS_PER_SEC));

	// --- Test 1 -------------------------------------------------------------------------
	printf("Test 1: plain C-Array with macro!\n"); 
	beginTimer = clock();      // start the timer
	for (int t=0;t<1000000;t++){
		test1[check(t)]=5;
	}
	endTimer = clock();
	printf("Timer: %f seconds.\n",((double)(endTimer - beginTimer)/(double)CLOCKS_PER_SEC));
	
	// --- Test2 -------------------------------------------------------------------------
	printf("Test 2: F-Array reserves dynamically up to 10000000 doubles! No copying, but memory not continuous\n");
	beginTimer = clock();      // start the timer
	for (int t=0;t<1000000;t++){
		test2[t]=5;
	}
	endTimer = clock();
	printf("Timer: %f seconds.\n",((double)(endTimer - beginTimer)/(double)CLOCKS_PER_SEC));

	// --- Test 3 ------------------------------------------------------------------------
	printf("Test 3: vector-Array reserves dynamically (not limited) doubles!Copies but memory is continuous\n"); 
	beginTimer = clock();      // start the timer
	for (int t=0;t<1000000;t++){
		test3[t]=5;
	}
	endTimer = clock();
	printf("Timer: %f seconds.\n",((double)(endTimer - beginTimer)/(double)CLOCKS_PER_SEC));


	printf("test3[1000]=%f\n",test3[1000]);
}
